#!/usr/bin/python
#
# enhanced caget
import Ezca, sys, getopt, types

doc = """
 caget (enhanced)
 Syntax: caget [options] PV1 PV2 ...

 for Process Variables PV1, PV2, etc.
 
 Options:
  -f "%9.4f"  or      --format="%9.4f"
     specify output format, using format strings.
  
  -t "mbbi" or        --type="full"
     force output based on known variable 'type'.
     currently supported enhanced types are "scalar"
     the default), "mbbi", "bo", and "full".

       'scalar': append '.VAL' suffix if not included
                 and append '.DESC' to get description
                 for output
       'bo':     PV is a binary out, lookup '.ZNAM'
                 or '.ONAM' field as appropriate
                 for the output value
       'mbbi':   PV is a mbbi, lookup '.ZRST', ...
                 as appropriate for the output value
       'full':   Do _NOT_ append '.VAL' and '.DESC' to PV.
                 (see also -r below).

     generally the 'bo' and 'mbbi' formats are not needed,
     as the appropropriate string value will be used, unless
     the -r switch is used.

  -r     or --raw
     force the exact PV name to be used, and supress any
     intelligence about returning the character version of
     the output.

"""

def show_usage():
    print doc
    sys.exit(1)

try:
    opts, args = getopt.getopt(sys.argv[1:],
                               "hert:f:",
                               ["help", "type=","format=","raw","enhanced"])
except:
    show_usage()
if (len(args) == 0):  show_usage()

pv_type = "scalar"
xform   = None
use_char= 1
raw     = 0
for key,val in opts:
    if key in ("-h", "--help"):
        show_usage()
        sys.exit()
    if key in ("-t", "--type"):
        pv_type = val
    if key in ("-f", "--format"):
        xform  = val
    if key in ("-r",  "--raw"):
        use_char =0
        raw      = 1
    if key in ("-e",  "--enhanced"):
        use_char = 1
        raw      = 0
        
ezca = Ezca.Ezca(timeout=0.0005, retrycount=300, use_char = use_char)

for pv in args:
    if (raw == 1):
        val = ezca.caget(pv)
        print pv, ' = ', val
    else:
        val,nam = ezca.getpv(pv, format=xform, type=pv_type)
        if (val == None):
            res = pv + " not found or not connected"
        elif (pv == nam):
            form = "%s = %13.7g"
            if (isinstance(val,types.StringType)):  form = "%s = %s"
            res =  form % (pv,val)
        else:
            form = "%s [%s] = %13.7g"
            if (isinstance(val,types.StringType)):  form = "%s [%s] = %s"
            res =  form % (pv,nam,val)        
        print res






